/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://eu-egee.org/partners/ for details on the copyright holders.
 * For license conditions see the license file or http://eu-egee.org/license.html
 *
 * Author Peter Kunszt <peter.kunszt@cern.ch>
 * $Id: TransferSRM.hxx,v 1.8 2005/12/19 11:24:26 mccance Exp $
 */

#ifndef GLITE_DATA_TRANSFER_SRM_HXX_
#define GLITE_DATA_TRANSFER_SRM_HXX_

#include "glite/data/transfer/TransferBase.hxx"
#include "glite/data/transfer/TransferFiles.hxx"
#include <vector>

#ifdef HAVE_FTIME
#undef HAVE_FTIME
#endif


namespace glite{
namespace transfer{

typedef struct TransferSRMFileStat_struct {
    int fileId;
    int status;
} TransferSRMFileStat;

typedef struct TransferSRMStat_struct {
    int status;
    char message[2048];
} TransferSRMStat;


class TransferArgsSRM : public TransferArgsBase {
public:

    TransferArgsSRM(TransferList *l) : TransferArgsBase(),
	m_list(l),
        m_srmcopy_timeout(120),   // this is only the timeout on the soap srmcp call
        m_endpoint_dest(true)
    {}

    const TransferList *  list() {return m_list;}
    int                   srmcopy_timeout()     {return m_srmcopy_timeout;}
    bool                  is_dest_endpoint()    {return m_endpoint_dest;}

    void setSRMCopyTimeout   (int srmcopy_timeout)    {m_srmcopy_timeout    = srmcopy_timeout;}
    void useDestEndpoint(bool v) {m_endpoint_dest = v;}
private:
    TransferArgsSRM() {}
    TransferList *        m_list;
    int                   m_srmcopy_timeout;
    bool                  m_endpoint_dest;
};		


/**
 * The TransferCommands is the main class to interact with the transfer.
 */

class TransferSRM : public TransferBase {
public:


    typedef enum {
	FILE_STATUS_PENDING = 0,
	FILE_STATUS_READY   = 1,
	FILE_STATUS_RUNNING = 2,
	FILE_STATUS_FAILED  = 3,
	FILE_STATUS_DONE    = 4,
	REQ_STATUS_PENDING  = 5,
	REQ_STATUS_READY    = 6,
	REQ_STATUS_ACTIVE   = 7,
	REQ_STATUS_FAILED   = 8,
	REQ_STATUS_DONE     = 9
    } SrmStatuses;


    /**
     * Client Transfer object based on an existing ID
     */
    TransferSRM(const std::string &id, int http_timeout = 40 );

    /**
     * Destructor
     */
    ~TransferSRM();

    /**
     * Spawn a transfer, retrieving the ID. This will not construct an
     * object but fork a separate process unless block is set to true,
     * in which case it blocks until the transfer is done.
     */
    static std::string start(TransferArgsSRM &args, 
			     std::vector<int> *fileId,
			     int (*change_procname)(const char *) = NULL);


    /**
     * retrieve transfer status
     */
    TransferSocket::Status status(std::vector<TransferSRMFileStat> *filestats, TransferSRMStat* transferstat,TransferLogger::Level loglevel = TransferLogger::INFO);

    /**
     * Abort transfer
     */
    void stop(TransferLogger::Level loglevel = TransferLogger::INFO);

    /**
     * Forget about this transfer (if SRM cannot be reached repeatedly for example)
     */
    void drop(TransferLogger::Level loglevel = TransferLogger::INFO);

protected:
    /**
     * Default Constructor disabled - have to get it via a known ID
     */
    TransferSRM() : TransferBase() {};

private:

    static void *monitor_transfer(void *arg);
    static void cleanup(const char *msg, bool fail);
    static void monitor_transfer_stop(pthread_t monitor_thread);
    static void copy_termination_handler(int signum);

    static void callSRMCopy(TransferArgsSRM & args,
			    TransferFiles & transfile,
			    std::string & endpoint
			    );
    static size_t getMapSize(int nfiles);
    static TransferLogger *s_logger;
    static int s_timeout;
    static int s_http_timeout;
    static time_t s_timeout_start;
    static std::string s_id;

    // non-static private member functions and variables
   
    int getREQState(char *state);
    int getState(char *filestate);
    void * readResult(std::vector<TransferSRMFileStat> *filestats, TransferSRMStat* transferstat, int nfiles, bool failed);
    TransferLogger *getLogger(TransferLogger::Level loglevel);
    TransferLogger *m_logger;
    int m_http_timeout;

};

} // namespace transfer
} // namespace glite

#endif //GLITE_DATA_TRANSFER_SRM_HXX_
